
# This is a comment

print   "Hello World!\n";

=pod

this is pod

=cut

Module->method();

my @words  = (
    qw/
         foo

         bar      baz               /,
    qw (
  hello there
    )       )  ;
  our @words2 = qw# one two three #;
__END__

=pod

more pod

=cut

end stuff
